//
//  LibytnefHelper.m
//  TNEFsEnough3
//
//  Created by Josh Jacob on 10/19/11.
//  Copyright 2011 Josh Jacob. All rights reserved.
//

#import "LibytnefHelper.h"
#import "ytnef.h"
#import "PrintTNEF.h"
#import "LibytnefFileItem.h"
#import "mapidefs.h"

@implementation LibytnefHelper

@synthesize parseResult;
@synthesize from;
@synthesize subject;
@synthesize date;
@synthesize body;
@synthesize bodyRTF;
@synthesize bodyHTML;
@synthesize bodyIsHTML;
@synthesize fileItems;

- (id) initWithFile: (NSURL *) file
{
    self = [super init];
    if (self) {
        
        cStringPath = nil;
        bodyIsHTML = NO;
        fileItems = [[NSMutableArray alloc] initWithCapacity:3];
        
        // get file path as c string
        NSString * pathString = [file path];
        cStringPath = malloc(sizeof(char) * [pathString length] + 2);
        [pathString getCString:cStringPath maxLength:([pathString length] + 2) encoding:NSUTF8StringEncoding];
        
        // init tnef structure
        TNEFInitialize(&tnefStruct);
        #ifdef DEBUG
        tnefStruct.Debug = 1;
        #endif
        
        [self parse];
    }
    
    return self;
}

- (void) dealloc
{
    // ytnef
    TNEFFree(&tnefStruct);
    
    // free
    if (cStringPath != nil)
        free(cStringPath);
    
    // release
    self.fileItems = nil;
    self.from = nil;
    self.subject = nil;
    self.date = nil;
    self.body = nil;
    
    [super dealloc];
}

#pragma mark - private methods

- (void) parse
{
    // parse file
    self.parseResult = TNEFParseFile(cStringPath, &tnefStruct);
    if (self.parseResult < 0)
    {
        // set error objects and return
        
        return;
    }
    
    // set from/subject/date fields
    if (tnefStruct.from.size > 0) {
        from = [[NSString alloc] initWithCString: tnefStruct.from.data encoding:NSWindowsCP1252StringEncoding];
    } else {
        variableLength * hasSender = MAPIFindProperty(&tnefStruct.MapiProperties, PROP_TAG(PT_UNICODE, PR_SENDER_EMAIL_ADDRESS));
        if (hasSender != MAPI_UNDEFINED) {
            from = [[NSString alloc] initWithCString: hasSender->data encoding:NSWindowsCP1252StringEncoding];
        }
    }
    if (tnefStruct.subject.size > 0) {
        subject = [[NSString alloc] initWithCString: tnefStruct.subject.data encoding:NSWindowsCP1252StringEncoding];
    }
    if (tnefStruct.dateSent.wYear > 0) {
        NSDateComponents * comps = [[NSDateComponents alloc] init];
        [comps setHour:tnefStruct.dateSent.wHour];
        [comps setMinute:tnefStruct.dateSent.wMinute];
        [comps setSecond:tnefStruct.dateSent.wSecond];
        [comps setDay:tnefStruct.dateSent.wDay];
        [comps setMonth:tnefStruct.dateSent.wMonth];
        [comps setYear:tnefStruct.dateSent.wYear];
        NSCalendar *gregorian = [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
        self.date = [gregorian dateFromComponents:comps];
        [gregorian release];
        [comps release];
    }
    
    // set body
    variableLength * hasBody = MAPIFindProperty(&tnefStruct.MapiProperties, PROP_TAG(PT_BINARY, PR_RTF_COMPRESSED));
    if (hasBody != MAPI_UNDEFINED) {
        if (IsCompressedRTF(hasBody) == 1)
        {
            int size;
            unsigned char * rawBody = DecompressRTF(hasBody, &size);
            
            NSData *data;
            
            data = [[NSData alloc] initWithBytes:rawBody length:size];
            // NSMutableDictionary * dict = [[NSMutableDictionary alloc] init];
            // [dict setValue:[NSNumber numberWithInt:NSWindowsCP1252StringEncoding] forKey:NSCharacterEncodingDocumentAttribute];
            
#ifdef __MAC_OS_X_VERSION_MAX_ALLOWED
            body = [[NSAttributedString alloc] initWithRTF:data documentAttributes: nil];
#endif
#ifndef __MAC_OS_X_VERSION_MAX_ALLOWED
            bodyRTF = [[NSData alloc] initWithData:data];
#endif
            
            [data release];
            free(rawBody);
            
        } else {
            
        }
    } else {
        
        variableLength * hasBody = MAPIFindProperty(&tnefStruct.MapiProperties, PROP_TAG(PT_BINARY, PR_BODY_HTML));
        if (hasBody != MAPI_UNDEFINED) {
            
            bodyIsHTML = YES;
            NSData *data;
            data = [[NSData alloc] initWithBytes:hasBody->data length:hasBody->size];
            
            NSString * htmlString = [[NSString alloc] initWithData:data encoding:NSISOLatin1StringEncoding];
            self.bodyHTML = htmlString;
            
            [htmlString release];
            [data release];
        }
    }
    
    // get embedded files
    
    Attachment *p = tnefStruct.starting_attach.next;
    while (p != NULL) {
        
        int i; //, object;
        char ifilename[256];
        variableLength *filename;
        //        variableLength *filedata;
        
        if  (p->RenderData.atyp == 1)
        {
            // case 1: printf("File      "); break;
            
            //            if (p->Title.size > 0) 
            //                DLog("%s", p->Title.data);
            //            DLog("\n");
            
            //            if (p->RenderData.dwFlags == 0x00000001) 
            //                printf("     MAC Binary Encoding\n");
            //            if (p->TransportFilename.size >0) 
            //                printf("     Transported under the name %s\n", p->TransportFilename.data);
            
            //            if (p->Date.wYear >0 ) {
            //                printf("    Date: ");
            //                TNEFPrintDate(p->Date);
            //                printf("\n");
            //            }
            //            if (p->CreateDate.wYear > 0) {
            //                printf("    Creation Date: ");
            //                TNEFPrintDate(p->CreateDate);
            //                printf("\n");
            //            }
            //            if (p->ModifyDate.wYear > 0) {
            //                printf("    Modified on: ");
            //                TNEFPrintDate(p->ModifyDate);
            //                printf("\n");
            //            }
            
            //            if (p->MAPI.count>0) {
            //                printf("    MAPI Properties: %i\n", p->MAPI.count);
            //                //if (verbose == 1) {
            //                //    MAPIPrint(&p->MAPI);
            //                //}
            //            }
            
            if (p->FileData.size > 0) {
                // DLog("    Attachment Size:  %ib\n", p->FileData.size);
                
                if ((filename = MAPIFindProperty(&(p->MAPI), PROP_TAG(30,0x3707))) == MAPI_UNDEFINED) {
                    if ((filename = MAPIFindProperty(&(p->MAPI), PROP_TAG(30,0x3001))) == MAPI_UNDEFINED) {
                        filename = &(p->Title);
                    }
                }
                
                sprintf(ifilename, "%s", filename->data);
                for(i=0; i<strlen(ifilename); i++) 
                    if (ifilename[i] == ' ') 
                        ifilename[i] = '_';
                // DLog("    File saves as [%s]\n", ifilename);
                
                
                // fill in table item
                LibytnefFileItem * fileItem = [[LibytnefFileItem alloc] init];
                fileItem.numBytes = p->FileData.size;
                fileItem.attachment = p;
                if (p->Title.size > 0) {
                    NSString * filenameNS = [[NSString alloc] initWithCString:ifilename encoding:NSWindowsCP1252StringEncoding];
                    fileItem.fileName = filenameNS;
                    filenameNS = nil;
                }
                if (fileItem.fileName == nil) {
                    fileItem.fileName = @"Untitled Attachment";
                }
                [self.fileItems addObject: fileItem];
                [fileItem release];
                
            }
        }
        p=p->next;
    }
    
    #ifdef DEBUG
    PrintTNEF(tnefStruct);
    #endif
}

@end
